
# indigoPlugin	- a Python scripted shader	for Maya
#
# While this shader doesn't really do anything special
# in Maya itself,	it's a convenient	way to present	the
# parameters needed for	Indigo materials to the	user,	
# and	to	pick up those parameters in the MtI	exporter.
#
# (c)	Doug Hammond 2008
# THE	SOFTWARE	IS	PROVIDED	"AS IS",	WITHOUT WARRANTY OF ANY	KIND.	
# THE	AUTHOR DOES	NOT MAKE	AND HEREBY DISCLAIMS	ANY EXPRESS	OR	IMPLIED 
# WARRANTIES INCLUDING,	BUT NOT LIMITED TO, THE	WARRANTIES OF 
# NON-INFRINGEMENT, MERCHANTABILITY	OR	FITNESS FOR	A PARTICULAR 
# PURPOSE, OR ARISING FROM	A COURSE	OF	DEALING,	USAGE, OR 
# TRADE PRACTICE.	IN	NO	EVENT	WILL THE	AUTHOR
# BE LIABLE	FOR ANY LOST REVENUES, DATA, OR PROFITS, OR SPECIAL, 
# DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES, EVEN	IF	THE AUTHOR 
# HAS	BEEN ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES.
#
# $Id: indigoPlugin.py 208	2008-03-22 13:14:12Z	doug $
#+

import sys,	math
import maya.OpenMaya	as	OpenMaya
import maya.OpenMayaRender	as	OpenMayaRender
import maya.OpenMayaUI as OpenMayaUI
import maya.OpenMayaMPx	as	OpenMayaMPx

# ------------------------------------------------------------------------------

# initialize the script	plug-ins
def initializePlugin(mobject):
	mplugin	= OpenMayaMPx.MFnPlugin(mobject)
	try:
		# indigoShader
		mplugin.registerNode(	indigoShader.nodeName(),
										indigoShader.nodeId(),
										indigoShader.nodeCreator,
										indigoShader.nodeInitializer,
										OpenMayaMPx.MPxNode.kDependNode,
										indigoShader.nodeClassify() )
										
		# indigoExitPortalLocator
		mplugin.registerNode(	indigoExitPortalLocator.nodeName(),
										indigoExitPortalLocator.nodeId(),
										indigoExitPortalLocator.nodeCreator,
										indigoExitPortalLocator.nodeInitializer,
										OpenMayaMPx.MPxNode.kLocatorNode	)
#			textFieldNode
		mplugin.registerNode(	textFieldNode.nodeName(),
										textFieldNode.nodeId(),
										textFieldNode.nodeCreator,
										textFieldNode.nodeInitializer,
										OpenMayaMPx.MPxNode.kDependNode,
										textFieldNode.nodeClassify() )
		# spectrumNode
		mplugin.registerNode(	spectrumNode.nodeName(),
										spectrumNode.nodeId(),
										spectrumNode.nodeCreator,
										spectrumNode.nodeInitializer,
										OpenMayaMPx.MPxNode.kDependNode,
										spectrumNode.nodeClassify() )
										
	except:
		sys.stderr.write( "Failed to register Indigo Plugin")
		raise

# uninitialize	the script plug-in
def uninitializePlugin(mobject):
	mplugin	= OpenMayaMPx.MFnPlugin(mobject)
	try:
		# indigoShader
		mplugin.deregisterNode( indigoShader.nodeId()	)
		# indigoExitPortalLocator
		mplugin.deregisterNode( indigoExitPortalLocator.nodeId() )
		# textFieldNode
		mplugin.deregisterNode( textFieldNode.nodeId() )
		# spectrumNode
		mplugin.deregisterNode( spectrumNode.nodeId()	)
		
	except:
		sys.stderr.write( "Failed to deregister	Indigo Plugin"	)
		raise

# ------------------------------------------------------------------------------

class	indigoExitPortalLocator(OpenMayaMPx.MPxLocatorNode):

	@staticmethod
	def nodeName():
		return	"indigoExitPortal"

	@staticmethod
	def nodeId():
		return	OpenMaya.MTypeId(0x6D746902)
	
	@staticmethod
	def nodeInitializer():
		return	OpenMaya.MStatus.kSuccess

	@staticmethod
	def nodeCreator():
		return	indigoExitPortalLocator()
						
	def __init__(self):
		OpenMayaMPx.MPxLocatorNode.__init__(self)
		
		glRenderer =	OpenMayaRender.MHardwareRenderer.theRenderer()
		self.glFT	= glRenderer.glFunctionTable()	
		
	@staticmethod		
	def isBounded():
		return	True;
		
	def boundingBox(self):
		bbox =	OpenMaya.MBoundingBox()
		bbox.expand(	OpenMaya.MPoint( -0.5,	0.0, -0.5 )	)
		bbox.expand(	OpenMaya.MPoint(	0.5,	0.0, -0.5 )	)
		bbox.expand(	OpenMaya.MPoint(	0.5,	0.0,	0.5 )	)
		bbox.expand(	OpenMaya.MPoint( -0.5,	0.0,	0.5 )	)
		bbox.expand(	OpenMaya.MPoint(	0.0, -0.5,	0.0 )	)
		bbox.expand(	OpenMaya.MPoint(	0.0,	0.0,	0.0 )	)
		return	bbox
		
	def draw(self, view, DGpath,	style, status):
		
		try:
				col =	self.colorRGB(	status )
				
				view.beginGL()
				self.glFT.glPushAttrib(	OpenMayaRender.MGL_CURRENT_BIT )
				
				self.glFT.glColor3f(	1,	0,	0 )
				self.glFT.glBegin( OpenMayaRender.MGL_LINES )
				self.glFT.glVertex3f( 0.0,	-0.5,	0.0 )
				self.glFT.glVertex3f( 0.0,	0.0,	0.0 )
				self.glFT.glEnd()
				
				self.glFT.glEnable( OpenMayaRender.MGL_BLEND	)
				self.glFT.glBlendFunc( OpenMayaRender.MGL_SRC_ALPHA, OpenMayaRender.MGL_ONE_MINUS_SRC_ALPHA )
				
				if	status == OpenMayaUI.M3dView.kLead:
					self.glFT.glColor4f( 0, 1, 0, 0.3 )
				else:
					self.glFT.glColor4f( col.r, col.g,	col.b, 0.3 )
				
				self.glFT.glBegin( OpenMayaRender.MGL_QUADS )
				self.glFT.glVertex3f( -0.5, 0.0,	-0.5 )
				self.glFT.glVertex3f(0.5, 0.0,	-0.5 )
				self.glFT.glVertex3f(0.5, 0.0,	0.5 )
				self.glFT.glVertex3f( -0.5, 0.0,	0.5 )
				self.glFT.glEnd()
				self.glFT.glDisable(	OpenMayaRender.MGL_BLEND )
				
				self.glFT.glPopAttrib()
				view.endGL()
		except:
				sys.stderr.write("Failed to draw	indigoExitPortal\n")
				raise
				
				
# ------------------------------------------------------------------------------

class	spectrumNode(OpenMayaMPx.MPxNode):

	@staticmethod
	def nodeName():
		return	"spectrumNode"

	@staticmethod
	def nodeId():
		return	OpenMaya.MTypeId(0x6D746A01)
		
	@staticmethod
	def nodeCreator():
		return	OpenMayaMPx.asMPxPtr( spectrumNode() )
				
	@staticmethod
	def nodeClassify():
		return	"utility/color"

	def __init__(self):
		OpenMayaMPx.MPxNode.__init__(self)
		aOutColor												= OpenMaya.MObject()
		spectrumType												= OpenMaya.MObject()
		
		# peak	attributes
		peak_min									= OpenMaya.MObject()
		peak_width								=	OpenMaya.MObject()
		base_value								=	OpenMaya.MObject()
		peak_value								=	OpenMaya.MObject()
		
		# blackbody attribute
		temperature								= OpenMaya.MObject()
		gain								=	OpenMaya.MObject()
		
		# rgb attributes
#			rgb								=	OpenMaya.MObject()
#			gamma									= OpenMaya.MObject()
		
		# uniform	attributes
		value								= OpenMaya.MObject()
		
		# regular	tabulated
 #			startWavelength								=	OpenMaya.MObject()
 #			endWavelength								= OpenMaya.MObject()
#		numValues								=	OpenMaya.MObject()
#		valArray									= OpenMaya.MObject()
		
#	def	compute(self, plug, block):
#			if	plug == self.aOutColor or plug.parent() == self.aOutColor:
		
				# THIS IS A	VERY SIMPLE	FLAT COLOUR	SHADER.
				# I CAN'T GET THE	LAMBERTIAN BIT	TO	WORK
				
#				resultColor =	block.inputValue(	self.common_Colour ).asFloatVector()
				
				
#				outColorHandle =	block.outputValue( self.aOutColor )
#				outColorHandle.setMFloatVector(resultColor)
#				outColorHandle.setClean()
#			else:
#				return OpenMaya.kUnknownParameter	
	
	@staticmethod
	def nodeInitializer():
		nAttr			=	OpenMaya.MFnNumericAttribute()
		lAttr			=	OpenMaya.MFnLightDataAttribute()
		enumAttr		= OpenMaya.MFnEnumAttribute()
		tAttr			= OpenMaya.MFnTypedAttribute()
		gAttr			= OpenMaya.MFnGenericAttribute()
	
		
		try:
		
				spectrumNode.aOutColor	= nAttr.createColor("outColor", "oc")
				nAttr.setKeyable(0)
				nAttr.setStorable(0)
				nAttr.setReadable(1)
				nAttr.setWritable(0)
				
				spectrumNode.spectrumType = enumAttr.create("spectrum", "spc",		0)
				#	PEAK
				enumAttr.addField( "peak",										2)
				enumAttr.addField( "blackbody",								0)
#				enumAttr.addField( "rgb",											1)
				enumAttr.addField( "uniform",										1)
 #				enumAttr.addField( "regular tabulated",					3)

				spectrumNode.peak_min =	nAttr.create("peak_min", "pkmn",	OpenMaya.MFnNumericData.kFloat, 380.00)
				spectrumNode.peak_width	= nAttr.create("peak_width", "pkwd", OpenMaya.MFnNumericData.kFloat,	1.0)
				spectrumNode.base_value	= nAttr.create("base_value", "pkbv", OpenMaya.MFnNumericData.kFloat,	1.0)
				spectrumNode.peak_value	= nAttr.create("peak_value", "pkpv", OpenMaya.MFnNumericData.kFloat,	0.0)
				
				spectrumNode.temperature =	nAttr.create("temperature", "t",	OpenMaya.MFnNumericData.kFloat, 6500.0)
				spectrumNode.gain	=	nAttr.create("gain", "g",	OpenMaya.MFnNumericData.kFloat, 0.01)				
				
#				spectrumNode.rgb	= nAttr.createColor("rgb","rgb")
#				makeOrdinary(	nAttr	)
#				nAttr.setUsedAsColor(1)
#				spectrumNode.gamma = nAttr.create("gamma", "gm", OpenMaya.MFnNumericData.kFloat, 2.2)
				
				spectrumNode.value =	nAttr.create("value","uv",	OpenMaya.MFnNumericData.kFloat, 0.5)
				
#				spectrumNode.startWavelength= nAttr.create("startWavelength","sw", OpenMaya.MFnNumericData.kFloat)
#				spectrumNode.endWavelength= nAttr.create("endWavelength","ew", OpenMaya.MFnNumericData.kFloat)
#				spectrumNode.numValues= nAttr.create("numValues","nv", OpenMaya.MFnNumericData.kLong)
#				spectrumNode.valArray=	nAttr.create("valArray","va",	OpenMaya.MFnNumericData.kFloat)

#OpenMaya.MFloatArray ?
		
				
#				makeOrdinary(	nAttr	)
#				nAttr.setUsedAsColor(1)
#				nAttr.setDefault(0.0, 0.0, 0.0)
#				makeOrdinary(	nAttr	)
#				makeOrdinary(	nAttr	)
#				nAttr.setDefault( 1500.0 )
				
		except:
				sys.stderr.write("Failed to create attributes in 'spectrumNode'\n")
				raise
				
		try:
				spectrumNode.addAttribute(spectrumNode.aOutColor)
				spectrumNode.addAttribute(spectrumNode.spectrumType)
				
				#uniform
				spectrumNode.addAttribute(spectrumNode.value)
				
				#rgb
#				spectrumNode.addAttribute(spectrumNode.rgb)
#				spectrumNode.addAttribute(spectrumNode.gamma)
				
				#blackbody
				spectrumNode.addAttribute(spectrumNode.temperature)
				spectrumNode.addAttribute(spectrumNode.gain)

				#peak	attrs
				spectrumNode.addAttribute(spectrumNode.peak_min)
				spectrumNode.addAttribute(spectrumNode.peak_width)
				spectrumNode.addAttribute(spectrumNode.base_value)
				spectrumNode.addAttribute(spectrumNode.peak_value)
								
				#tabulated data
#				spectrumNode.addAttribute(spectrumNode.startWavelength)
#				spectrumNode.addAttribute(spectrumNode.endWavelength)
#				spectrumNode.addAttribute(spectrumNode.numValues)
#				spectrumNode.addAttribute(spectrumNode.valArray)
				
		except:
				sys.stderr.write("Failed to add attributes in spectrumNode\n")
				raise
	
#			try:
#				#	Diffuse Material
##				spectrumNode.attributeAffects (spectrumNode.rgb,			spectrumNode.aOutColor)
#	#		spectrumNode.attributeAffects (spectrumNode.common_BumpMapping,		spectrumNode.aOutColor)
#				
#			except:
#				sys.stderr.write("Failed in setting attributeAffects	in	spectrumNode\n")
#				raise
		
		
#-------------------------------------------------------------------------------

class	textFieldNode(OpenMayaMPx.MPxNode):
	
	@staticmethod
	def nodeName():
		return	"textFieldNode"
	
	@staticmethod
	def nodeId():
		return	OpenMaya.MTypeId(0x6D746900)
		
	@staticmethod
	def nodeCreator():
		return	OpenMayaMPx.asMPxPtr( textFieldNode() )
	
	@staticmethod
	def nodeClassify():
		return	"utility/general"
	
	def __init__(self ):
		OpenMayaMPx.MPxNode.__init__(self)
		
		text =	OpenMaya.MObject()
		
#	def	compute(self, plug, block):
		
	@staticmethod
	def nodeInitializer():
		
		nAttr			=	OpenMaya.MFnNumericAttribute()
		lAttr			=	OpenMaya.MFnLightDataAttribute()
		enumAttr		= OpenMaya.MFnEnumAttribute()
		tAttr			= OpenMaya.MFnTypedAttribute()
		gAttr			= OpenMaya.MFnGenericAttribute()

		try:
				#textFieldNode.text = Attr.create ("text", "t",	OpenMaya.kString,	""	)
				#makeInput(	tAttr	)
				#tAttr.setDefault("Hello World !")
				
#				strData.create()	should be used	
#CODE
#MFnStringData	strData;
#aOutFilename = tAttr.create("outFilename", "of", MFnData::kString, strData.create());
				
				textFieldNode.aOutColor	=	nAttr.createColor("outColor",	"oc")
				nAttr.setKeyable(0)
				nAttr.setStorable(0)
				nAttr.setReadable(1)
				nAttr.setWritable(0)
		
		except:
				sys.stderr.write("Failed to create attributes in textFileNode\n")
				raise

		try:
				textFieldNode.addAttribute(textFieldNode.aOutColor) 
		
		except:
			sys.stderr.write("Failed	to	add attributes\n")
			raise
# ------------------------------------------------------------------------------

class	indigoShader(OpenMayaMPx.MPxNode):

	@staticmethod
	def nodeName():
		return	"indigoShader"

	@staticmethod
	def nodeId():
		return	OpenMaya.MTypeId(0x6D746901)
		
	@staticmethod
	def nodeCreator():
		return	OpenMayaMPx.asMPxPtr( indigoShader() )
				
	@staticmethod
	def nodeClassify():
		return	"shader/surface"

	def __init__(self):
		OpenMayaMPx.MPxNode.__init__(self)
		
		iMaterialType			= OpenMaya.MObject()
		aOutColor				= OpenMaya.MObject()
		aOutTransparency		= OpenMaya.MObject()
#		aOutNormal			= OpenMaya.MObject()
		
		# Common attributes
		common_Colour			= OpenMaya.MObject()
		common_BumpMapping	= OpenMaya.MObject()
		common_Layer				= OpenMaya.MObject()
		baseEmission				= OpenMaya.MObject()
		emission					= OpenMaya.MObject()
		common_Transparency	= OpenMaya.MObject()
		
		# Diffuse	material
		# diffuse_AlbedoSpectrum	-	use common_Colour
		diffuse_Transmitter	= OpenMaya.MObject()
		
		# Oren-Nayar	material
		# orenNayar_AlbedoSpectrum -use	common_Colour
		orenNayar_sigma		= OpenMaya.MObject()
		
		# Phong material
		# phong_DiffuseAlbedoSpectrum - use common_Colour
		phong_IOR							= OpenMaya.MObject()
		phong_Exponent					= OpenMaya.MObject()
#		phong_NKData						= OpenMaya.MObject()
#		phong_ExponentMapABC			= OpenMaya.MObject()
#		phong_ExponentMapExp			= OpenMaya.MObject()
		phong_SpecularReflectivity		= OpenMaya.MObject()
		
		
		# Medium
		# medium_AbsorptionCoefficientSpectrum - use Common_Colour
		medium_Precedence									= OpenMaya.MObject()
		medium_Type											= OpenMaya.MObject()
		medium_MelaninFraction								= OpenMaya.MObject()
		medium_MelaninTypeBlend							= OpenMaya.MObject()
		medium_HemoglobinFraction							= OpenMaya.MObject()
		medium_IOR											= OpenMaya.MObject()
		medium_CauchyB										= OpenMaya.MObject()
		medium_Density										= OpenMaya.MObject()
		medium_SubsurfaceScattering						= OpenMaya.MObject()
		medium_SubsurfaceScatteringCoefficientSpectrum	= OpenMaya.MObject()
		medium_SubsurfaceScatteringPhaseFunction			= OpenMaya.MObject()
		medium_SubsurfaceScatteringGSpectrum			= OpenMaya.MObject()
		
		# Specular
		specular_Transparent				= OpenMaya.MObject()
		absorption_layer_transmittance	= OpenMaya.MObject()
		
		# Glossy Transparent
		glossyTransparent_Exponent		=	OpenMaya.MObject()
		
		
	def compute(self, plug, block):

#			if	(plug	== aOutColor
#				|| plug	==	aOutColorR
 #				|| plug	==	aOutColorG
 #				|| plug	==	aOutColorB
 #				|| plug	==	aOutTransparency
#				|| plug	==	aOutTransR
#				|| plug	==	aOutTransG
#				|| plug	==	aOutTransB )
	
		if plug == self.aOutColor or plug.parent()	==	self.aOutColor:
		
				# THIS IS A	VERY SIMPLE	FLAT COLOUR	SHADER.
				# I CAN'T GET THE	LAMBERTIAN BIT	TO	WORK
				
				resultColor	= block.inputValue( self.common_Colour	).asFloatVector()
				
				
				outColorHandle	= block.outputValue(	self.aOutColor	)
				outColorHandle.setMFloatVector(resultColor)
				outColorHandle.setClean()
		else:
				return OpenMaya.kUnknownParameter	
	
	@staticmethod
	def nodeInitializer():
		nAttr			= OpenMaya.MFnNumericAttribute()
		lAttr			= OpenMaya.MFnLightDataAttribute()
		enumAttr		= OpenMaya.MFnEnumAttribute()
		tAttr			= OpenMaya.MFnTypedAttribute()
		gAttr			= OpenMaya.MFnGenericAttribute()
	
		try:
		
				indigoShader.iMaterialType	= enumAttr.create("material",	"mat",	0)
				enumAttr.addField( "Diffuse", 0)
				enumAttr.addField( "Phong", 1)
				enumAttr.addField( "Oren-Nayar", 2)
				enumAttr.addField( "Specular", 3)
				enumAttr.addField( "Glossy Transparent", 4)
				enumAttr.addField( "Null", 5)
				#enumAttr.addField( "Blend", n)
	
				indigoShader.aOutColor	= nAttr.createColor("outColor", "oc")
				nAttr.setKeyable(0)
				nAttr.setStorable(0)
				nAttr.setReadable(1)
				nAttr.setWritable(0)
	
				indigoShader.aOutTransparency	=	nAttr.createColor("outTranspareny",	"ot")
				nAttr.setKeyable(0)
				nAttr.setStorable(0)
				nAttr.setReadable(1)
				nAttr.setWritable(0)
	
#				indigoShader.aOutNormal	=	nAttr.createPoint("outNormal", "on")
#				nAttr.setKeyable(0)
#				nAttr.setStorable(0)
#				nAttr.setReadable(1)
#				nAttr.setWritable(0)
				
				# Common	attributes
				indigoShader.common_Colour	= nAttr.createColor("color", "c")
				makeInput( nAttr )
				nAttr.setUsedAsColor(1)
				nAttr.setDefault(0.8, 0.8,	0.8)
				#-
 #				indigoShader.common_AlbedoTextureABC = nAttr.createColor("colorABC", "cabc")
 #				makeOrdinary(	nAttr	)
#			nAttr.setUsedAsColor(0)
#			nAttr.setDefault(0.0, 1.0, 0.0)
				#-
	#			indigoShader.common_AlbedoTextureExp = nAttr.create("colourExponent", "ce", OpenMaya.MFnNumericData.kFloat)
	#		makeOrdinary(	nAttr	)
	#		nAttr.setDefault( 2.2 )
				
				#-	normalCamera hidden,	allows bump	mapping
				indigoShader.common_BumpMapping = nAttr.createPoint("normalCamera","n")
				makeInput( nAttr )
				nAttr.setHidden(1)
				nAttr.setDefault(0, 0, 1.0)
				#-
 #				indigoShader.common_BumpMapABC = nAttr.createColor("bumpABC",	"babc")
 #				makeOrdinary(	nAttr	)
 #				nAttr.setUsedAsColor(0)
	#			nAttr.setDefault(0.0, 1.0, 0.0)
				#-
#				indigoShader.common_BumpMapExp = nAttr.create("bumpExponent",	"be",	OpenMaya.MFnNumericData.kFloat)
#				makeOrdinary(	nAttr	)
#				nAttr.setDefault( 1.0 )
				#-
				indigoShader.common_Transparency	= nAttr.createColor("transparency","t")
				makeInput( nAttr )
				nAttr.setDefault(0.0, 0.0,	0.0)
				#-
				indigoShader.common_Layer = nAttr.create("lightLayer", "ll", OpenMaya.MFnNumericData.kInt)
				makeOrdinary( nAttr )
				#-
				indigoShader.baseEmission = nAttr.createColor("ambientColor", "bem")
				makeInput( nAttr )
				nAttr.setUsedAsColor(1)
				nAttr.setDefault(0.0, 0.0,	0.0)
				#-
				indigoShader.emission =	nAttr.createColor("incandescence", "em")
				makeOrdinary( nAttr )
				nAttr.setUsedAsColor(1)
				nAttr.setDefault(0.0, 0.0,	0.0)
				
				# Diffuse Material
				indigoShader.diffuse_Transmitter	= nAttr.create("diffuseTransmitter", "dt", OpenMaya.MFnNumericData.kBoolean)
				makeOrdinary( nAttr )
				
				
				# Oren-Nayar Material
				indigoShader.orenNayar_sigma = nAttr.create("orenNayarSigma", "ons",	OpenMaya.MFnNumericData.kFloat)
				makeOrdinary( nAttr )
				nAttr.setDefault(	0.2 )
				
				
				# Phong Material
				#-
				indigoShader.phong_Type	= enumAttr.create("specType",	"spt",	0)
				
				enumAttr.addField( "ior",									0)
				enumAttr.addField( "specular reflectivity",											1)
				enumAttr.addField( "nkData",								2)
				
				indigoShader.phong_IOR = nAttr.create("phongIOR", "pior", OpenMaya.MFnNumericData.kFloat)
				makeOrdinary( nAttr )
				nAttr.setDefault(	1.25 )
				#-
				indigoShader.phong_Exponent =	nAttr.create("cosinePower", "pe", OpenMaya.MFnNumericData.kFloat)
				makeOrdinary( nAttr )
				nAttr.setDefault(	1500.0 )
				#-
#				indigoShader.phong_NKData	= enumAttr.create("nkData", "nk",	0)! mti_addMatAttrs !
				#-
#				indigoShader.phong_ExponentMapABC = nAttr.createColor("phongExponentABC",	"peabc")
#				makeOrdinary(	nAttr	)
#				nAttr.setUsedAsColor(0)
#				nAttr.setDefault(0.0, 1.0, 0.0)
				#-
#				indigoShader.phong_ExponentMapExp = nAttr.create("phongExponentMapExponent",	"peme", OpenMaya.MFnNumericData.kFloat)
#				makeOrdinary(	nAttr	)
#				nAttr.setDefault( 1.0 )
				#-
				indigoShader.phong_SpecularReflectivity =	nAttr.createColor("phongSpecularReflectivity", "psrs")
				makeInput( nAttr )
				nAttr.setUsedAsColor(1)
				nAttr.setDefault(0.0, 0.0,	0.0)
				#-
	
				
				# Medium
				indigoShader.medium_Precedence =	nAttr.create("mediumPrecedence",	"mp",	OpenMaya.MFnNumericData.kInt)
				makeOrdinary( nAttr )
				nAttr.setDefault(	6 )
				#-
				indigoShader.medium_Type =	enumAttr.create("mediumType",	"mt",		0)
				enumAttr.addField( "Basic",				0)			
				enumAttr.addField( "Dermis",			1)
				enumAttr.addField( "Epidermis",			2)
				#-
				indigoShader.medium_MelaninFraction	= nAttr.create("mediumMelaninFraction", "mmf", OpenMaya.MFnNumericData.kFloat)
				makeOrdinary( nAttr )
				nAttr.setDefault(	0.25 )
				#-
				indigoShader.medium_MelaninTypeBlend =	nAttr.create("mediumMelaninTypeBlend",	"mmtb", OpenMaya.MFnNumericData.kFloat)
				makeOrdinary( nAttr )
				nAttr.setDefault(	0.5 )
				#-
				indigoShader.medium_HemoglobinFraction	= nAttr.create("mediumHemoglobinFraction", "mhf", OpenMaya.MFnNumericData.kFloat)
				makeOrdinary( nAttr )
				nAttr.setDefault(	0.01 )
				#-
				indigoShader.medium_IOR	= nAttr.create("mediumIOR", "mior",	OpenMaya.MFnNumericData.kFloat)
				makeOrdinary( nAttr )
				nAttr.setDefault(	1.333	)
				#-
				indigoShader.medium_CauchyB =	nAttr.create("mediumCauchyB",	"mcb", OpenMaya.MFnNumericData.kFloat)
				makeOrdinary( nAttr )
				nAttr.setDefault(	0.0 )
				#-
				# added by Hellstorm
				indigoShader.medium_Density =	nAttr.create("mediumDensity","md", OpenMaya.MFnNumericData.kFloat)
				makeOrdinary( nAttr )
				nAttr.setDefault ( 1.0 )
				nAttr.setMin (	0.0 )
				#-
				indigoShader.medium_SubsurfaceScattering = nAttr.create("mediumSubsurfaceScattering", "msss", OpenMaya.MFnNumericData.kBoolean)
				makeInput( nAttr )
				#-
				indigoShader.medium_SubsurfaceScatteringCoefficientSpectrum	= nAttr.createColor("mediumSSSSpectrum", "mssss")
				makeOrdinary( nAttr )
				nAttr.setUsedAsColor(1)
				nAttr.setDefault(0.0, 0.0,	0.0)
				#-
				indigoShader.medium_SubsurfaceScatteringPhaseFunction	= enumAttr.create("mediumPhaseFunction", "mpf",		0)
				enumAttr.addField( "Uniform",						0)			
				enumAttr.addField( "Henyey-Greenstein",		1)
				indigoShader.medium_SubsurfaceScatteringGSpectrum = nAttr.createColor("mediumGSpectrum", "mgs")
				makeOrdinary( nAttr )
				nAttr.setUsedAsColor(1)
				nAttr.setDefault(0.0, 0.0,	0.0)
				
				# Specular
				indigoShader.specular_Transparent =	nAttr.create("specularTransparent",	"st",	OpenMaya.MFnNumericData.kBoolean)
				makeOrdinary( nAttr )				
#				nAttr.setDefault(1.0)
				#-
				indigoShader.absorption_layer_transmittance = nAttr.createColor("transmittanceLayer", "tl")
				
				# Glossy	Transparent
				indigoShader.glossyTransparent_Exponent =	nAttr.create("glossyTransparentExponent",	"ge",	OpenMaya.MFnNumericData.kFloat)
				makeOrdinary( nAttr )
				nAttr.setDefault(	1500.0 )
				
		except:
				sys.stderr.write("Failed to create attributes\n")
				raise
				
		try:
				indigoShader.addAttribute(indigoShader.iMaterialType)		
				indigoShader.addAttribute(indigoShader.aOutColor)
				indigoShader.addAttribute(indigoShader.aOutTransparency)
#				indigoShader.addAttribute(indigoShader.aOutNormal)
				
				#Common attrs
				indigoShader.addAttribute(indigoShader.common_Colour)
				#indigoShader.addAttribute(indigoShader.common_AlbedoTextureABC)
				#indigoShader.addAttribute(indigoShader.common_AlbedoTextureExp)
				indigoShader.addAttribute(indigoShader.common_BumpMapping)
				indigoShader.addAttribute(indigoShader.common_Transparency)
				#indigoShader.addAttribute(indigoShader.common_BumpMapABC)
				#indigoShader.addAttribute(indigoShader.common_BumpMapExp)
				indigoShader.addAttribute(indigoShader.common_Layer)
				indigoShader.addAttribute(indigoShader.baseEmission)
				indigoShader.addAttribute(indigoShader.emission)
				
				#Diffuse	Material
				indigoShader.addAttribute(indigoShader.diffuse_Transmitter)
				
				#Oren-Nayar	Material
				indigoShader.addAttribute(indigoShader.orenNayar_sigma)
				
				#Phong Material
				indigoShader.addAttribute(indigoShader.phong_Type)
				indigoShader.addAttribute(indigoShader.phong_IOR)
				indigoShader.addAttribute(indigoShader.phong_Exponent)
				#indigoShader.addAttribute(indigoShader.phong_NKData)
#				indigoShader.addAttribute(indigoShader.phong_ExponentMapABC)
#				indigoShader.addAttribute(indigoShader.phong_ExponentMapExp)
				indigoShader.addAttribute(indigoShader.phong_SpecularReflectivity)
				
				# Medium
				indigoShader.addAttribute(indigoShader.medium_Precedence)
				indigoShader.addAttribute(indigoShader.medium_Type)
				indigoShader.addAttribute(indigoShader.medium_IOR)
				indigoShader.addAttribute(indigoShader.medium_CauchyB)
				indigoShader.addAttribute(indigoShader.medium_Density)
				indigoShader.addAttribute(indigoShader.medium_SubsurfaceScattering)
				indigoShader.addAttribute(indigoShader.medium_SubsurfaceScatteringCoefficientSpectrum)
				indigoShader.addAttribute(indigoShader.medium_SubsurfaceScatteringPhaseFunction)
				indigoShader.addAttribute(indigoShader.medium_SubsurfaceScatteringGSpectrum)
				indigoShader.addAttribute(indigoShader.medium_MelaninFraction)
				indigoShader.addAttribute(indigoShader.medium_MelaninTypeBlend)
				indigoShader.addAttribute(indigoShader.medium_HemoglobinFraction)
				
				# Specular
				indigoShader.addAttribute(indigoShader.specular_Transparent)
				indigoShader.addAttribute(indigoShader.absorption_layer_transmittance)
				
				# Glossy	Transparent
				indigoShader.addAttribute(indigoShader.glossyTransparent_Exponent)
				
		except:
				sys.stderr.write("Failed to add attributes\n")
				raise
	
		try:
				# Diffuse Material
				indigoShader.attributeAffects	(indigoShader.common_Colour,			indigoShader.aOutColor)
				indigoShader.attributeAffects	(indigoShader.common_Transparency,	indigoShader.aOutTransparency)
#				indigoShader.attributeAffects	(indigoShader.common_BumpMapping,	indigoShader.aOutNormal)
				
		except:
				sys.stderr.write("Failed in setting	attributeAffects\n")
				raise
		


		
def makeInput(attr):
	attr.setKeyable(1)
	attr.setStorable(1)
	attr.setReadable(1)
	attr.setWritable(1)
	
def makeOrdinary(attr):
	attr.setKeyable(1)
	attr.setStorable(1)
	attr.setReadable(0)
	attr.setWritable(1)
